import React, { useState, useCallback, useMemo } from "react";
import { Container, Row, Col } from "react-grid-system";
import { Model } from "@gooddata/react-components";
import ReactModal from "react-modal";

import Page from "../components/Page";
import C from "../constants";
import VisualizationBuilderWidget from "../components/visualizations/VisualizationBuilderWidget/VisualizationBuilderWidget";
import Drill from "../components/Drill";
import Button from "../components/controls/Button";
import { columnChartVisType } from "../components/visualizations/VisualizationBuilderWidget/visTypes";

import sharedStyles from "../shared.module.scss";

const visTypePreset = {
    ...columnChartVisType,
    buckets: [
        {
            ...columnChartVisType.buckets[0],
            items: [Model.measure(C.measures.totalSales)],
        },
        {
            ...columnChartVisType.buckets[1],
            items: [Model.attribute(C.attributeDisplayForms.locationState)],
        },
        columnChartVisType.buckets[2],
    ],
};

ReactModal.setAppElement("body");

//  This component edits a specified bucket and replaces it's last attribute,
// then filters the visualization by the same attribute value found in drillContext intersection
const VisualizationWithDrill = ({
    drillContext,
    visType,
    // an index of a bucket that should be replaced (viewBy; index 1)
    drillBucketIndex = 1,
    // what will be the attribute replaced by
    replacementAttributeIdentifier = C.attributeDisplayForms.monthShort,
}) => {
    if (!drillContext || !visType) {
        return null;
    }

    const originalBucket = visType.buckets[drillBucketIndex];
    const originalBucketItems = originalBucket.items;
    const originalAttribute = originalBucketItems[originalBucketItems.length - 1];
    const originalAttributeIdentifier = originalAttribute.visualizationAttribute.displayForm.identifier;

    const modalVisType = {
        ...visType,
        buckets: [...visType.buckets],
    };

    // replace items in a drillBucket
    modalVisType.buckets[drillBucketIndex] = {
        ...originalBucket,
        items: [...originalBucketItems],
    };
    const newBucketItems = modalVisType.buckets[drillBucketIndex].items;

    // replace last attribute with replacementAttribute
    newBucketItems[newBucketItems.length - 1] = Model.attribute(replacementAttributeIdentifier);

    // Find out attribute value of the drill intersection that matches original attribute
    const drillAttributeIntersectionPoint = drillContext.intersection.find(
        point => point.identifier === originalAttributeIdentifier,
    );

    // Filter visualization so that it shows only data that matches attribute value
    const drillFilter = Model.attributeFilter(originalAttributeIdentifier).in(
        drillAttributeIntersectionPoint.title,
    );

    return (
        <VisualizationBuilderWidget
            className={sharedStyles.ModalContent}
            headline={drillAttributeIntersectionPoint.title}
            visTypes={null}
            initialVisType={modalVisType}
            visualizationProps={{
                filters: [drillFilter],
            }}
        />
    );
};

const DrillToModal = () => {
    const [isModalOpen, setIsModalOpen] = useState(false);
    const [drillContext, setDrillContext] = useState(null);
    const onDrill = useCallback(newDrillContext => {
        setDrillContext(newDrillContext);
        setIsModalOpen(true);
    }, []);
    const on = useMemo(() => {
        return {
            [C.measures.totalSales]: onDrill,
        };
    }, [onDrill]);

    return (
        <Page>
            <div className={sharedStyles.Header}>
                <Container fluid xs sm md lg>
                    <Row>
                        <Col className={sharedStyles.Column}>
                            <h2 className={sharedStyles.Title}>DrillToModal</h2>
                        </Col>
                    </Row>
                </Container>
            </div>
            <Container fluid xs sm md lg>
                <Row>
                    <Col>
                        <Drill on={on}>
                            {drillingProps => {
                                return (
                                    <>
                                        <VisualizationBuilderWidget
                                            headline="Drill to Modal"
                                            visTypes={null}
                                            initialVisType={visTypePreset}
                                            visualizationProps={drillingProps}
                                        />
                                        <ReactModal
                                            isOpen={isModalOpen}
                                            onRequestClose={() => {
                                                setIsModalOpen(false);
                                            }}
                                            className={sharedStyles.Modal}
                                            overlayClassName={sharedStyles.ModalOverlay}
                                        >
                                            <Button
                                                onClick={() => setIsModalOpen(false)}
                                                className={sharedStyles.ModalClose}
                                            >
                                                ✕
                                            </Button>
                                            <VisualizationWithDrill
                                                visType={visTypePreset}
                                                drillContext={drillContext}
                                            />
                                        </ReactModal>
                                    </>
                                );
                            }}
                        </Drill>
                    </Col>
                </Row>
            </Container>
        </Page>
    );
};

export default DrillToModal;
